/*
 * Copyright 2004-2009 the Seasar Foundation and the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.seasar.doma.internal.jdbc.query;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.seasar.doma.internal.jdbc.entity.EntityPropertyType;
import org.seasar.doma.internal.jdbc.entity.EntityType;
import org.seasar.doma.internal.jdbc.entity.EntityTypeFactory;
import org.seasar.doma.internal.jdbc.entity.VersionPropertyType;
import org.seasar.doma.internal.jdbc.sql.PreparedSql;
import org.seasar.doma.internal.jdbc.util.ColumnUtil;
import org.seasar.doma.internal.jdbc.util.TableUtil;
import org.seasar.doma.internal.message.DomaMessageCode;
import org.seasar.doma.internal.util.AssertionUtil;
import org.seasar.doma.jdbc.Config;
import org.seasar.doma.jdbc.JdbcException;
import org.seasar.doma.jdbc.SqlExecutionSkipCause;

/**
 * @author taedium
 * 
 */
public abstract class AutoModifyQuery<E> implements ModifyQuery {

    protected static final String[] EMPTY_STRINGS = new String[] {};

    protected String[] includedPropertyNames = EMPTY_STRINGS;

    protected String[] excludedPropertyNames = EMPTY_STRINGS;

    protected final EntityTypeFactory<E> entityTypeFactory;

    protected Config config;

    protected EntityType<E> entityType;

    protected String callerClassName;

    protected String callerMethodName;

    protected PreparedSql sql;

    protected final List<EntityPropertyType<?>> targetProperties = new ArrayList<EntityPropertyType<?>>();

    protected final List<EntityPropertyType<?>> idProperties = new ArrayList<EntityPropertyType<?>>();

    protected VersionPropertyType<?> versionPropertyType;

    protected String tableName;

    protected final Map<String, String> columnNameMap = new HashMap<String, String>();

    protected boolean optimisticLockCheckRequired;

    protected boolean autoGeneratedKeysSupported;

    protected boolean executable;

    protected SqlExecutionSkipCause sqlExecutionSkipCause = SqlExecutionSkipCause.STATE_UNCHANGED;

    protected int queryTimeout;

    public AutoModifyQuery(EntityTypeFactory<E> entityTypeFactory) {
        AssertionUtil.assertNotNull(entityTypeFactory);
        this.entityTypeFactory = entityTypeFactory;
    }

    protected void prepareTableAndColumnNames() {
        tableName = TableUtil.getQualifiedTableName(entityType);
        for (EntityPropertyType<?> p : entityType.getEntityPropertyTypes()) {
            String columnName = ColumnUtil.getColumnName(entityType, p);
            columnNameMap.put(p.getName(), columnName);
        }
    }

    protected void prepareIdAndVersionProperties() {
        for (EntityPropertyType<?> p : entityType.getEntityPropertyTypes()) {
            if (p.isId()) {
                idProperties.add(p);
            }
        }
        versionPropertyType = entityType.getVersionPropertyType();
    }

    protected void validateIdExistent() {
        if (idProperties.isEmpty()) {
            throw new JdbcException(DomaMessageCode.DOMA2022, entityType
                    .getName());
        }
    }

    protected void prepareOptions() {
        if (queryTimeout <= 0) {
            queryTimeout = config.getQueryTimeout();
        }
    }

    protected boolean isTargetPropertyName(String name) {
        if (includedPropertyNames.length > 0) {
            for (String includedName : includedPropertyNames) {
                if (includedName.equals(name)) {
                    for (String excludedName : excludedPropertyNames) {
                        if (excludedName.equals(name)) {
                            return false;
                        }
                    }
                    return true;
                }
            }
            return false;
        }
        if (excludedPropertyNames.length > 0) {
            for (String excludedName : excludedPropertyNames) {
                if (excludedName.equals(name)) {
                    return false;
                }
            }
            return true;
        }
        return true;
    }

    @Override
    public void complete() {
        entityType.refreshEntity();
    }

    public void setConfig(Config config) {
        this.config = config;
    }

    public void setEntity(E entity) {
        this.entityType = entityTypeFactory.createEntityType(entity);
    }

    public void setCallerClassName(String callerClassName) {
        this.callerClassName = callerClassName;
    }

    public void setCallerMethodName(String callerMethodName) {
        this.callerMethodName = callerMethodName;
    }

    public void setQueryTimeout(int queryTimeout) {
        this.queryTimeout = queryTimeout;
    }

    public void setIncludedPropertyNames(String... includedPropertyNames) {
        this.includedPropertyNames = includedPropertyNames;
    }

    public void setExcludedPropertyNames(String... excludedPropertyNames) {
        this.excludedPropertyNames = excludedPropertyNames;
    }

    public PreparedSql getSql() {
        return sql;
    }

    public String getClassName() {
        return callerClassName;
    }

    public String getMethodName() {
        return callerMethodName;
    }

    public Config getConfig() {
        return config;
    }

    public boolean isOptimisticLockCheckRequired() {
        return optimisticLockCheckRequired;
    }

    @Override
    public boolean isExecutable() {
        return executable;
    }

    @Override
    public SqlExecutionSkipCause getSqlExecutionSkipCause() {
        return sqlExecutionSkipCause;
    }

    @Override
    public boolean isAutoGeneratedKeysSupported() {
        return autoGeneratedKeysSupported;
    }

    @Override
    public int getQueryTimeout() {
        return queryTimeout;
    }

    @Override
    public String toString() {
        return sql != null ? sql.toString() : null;
    }
}
